using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace TFramework
{
    public class EditorReferencePool
    {
        private static Dictionary<Type, List<IReference>> _referencePools = new Dictionary<Type, List<IReference>>();

        /// <summary>
        /// ȡ
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public static int GetRerenceCount(Type type)
        {
            if (_referencePools.ContainsKey(type))
                return _referencePools[type].Count;
            else
                throw new UnityException($"ȡʧܣó{type.FullName}");
        }

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <returns></returns>
        public static T Spawn<T>() where T : class, IReference, new()
        {
            Type type = typeof(T);
            RegisterRefer(type);
            T obj = null;
            if (GetRerenceCount(type) > 0)
            {
                obj = _referencePools[type][0] as T;
                _referencePools[type].Remove(obj);
            }
            else
                obj = new T();
            return obj;
        }

        /// <summary>
        /// ָ
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="predicate"></param>
        /// <returns></returns>
        public static T Spawn<T>(Predicate<T> predicate) where T : class, IReference, new()
        {
            Type type = typeof(T);
            RegisterRefer(type);
            T obj = null;
            if (GetRerenceCount(type) > 0)
            {
                IReference reference = _referencePools[type].Find(p=>(bool)predicate?.Invoke(p as T));
                if(reference!=null)
                {
                    obj = reference as T;
                    _referencePools[type].Remove(reference);
                }
                else
                    obj = new T();
            }
            else
                obj = new T();
            return obj;
        }

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="T"></typeparam>
        public static void Recycle<T>(T refer) where T : class, IReference, new()
        {
            Type type = refer.GetType();
            RegisterRefer(type);
            _referencePools[type].Add(refer);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="refes">ü</param>
        public static void Recycles<T>(List<T> refes) where T : class, IReference, new() => Recycles<T>(refes.ToArray());

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="refes"></param>
        public static void Recycles<T>(T[] refes) where T : class, IReference, new()
        {
            if (refes == null) return;
            Type type = typeof(T);
            RegisterRefer(type);
            for (int i = 0; i < refes.Length; i++)
                _referencePools[type].Add(refes[i]);
        }

        /// <summary>
        /// ָó
        /// </summary>
        /// <param name="type"></param>
        public static void Clear(Type type) => _referencePools[type].Clear();

        /// <summary>
        /// ó
        /// </summary>
        public static void ClearAll()
        {
            foreach (var refer in _referencePools)
                refer.Value.Clear();
        }

        /// <summary>
        /// ע
        /// </summary>
        /// <param name="type"></param>
        private static void RegisterRefer(Type type)
        {
            if (!_referencePools.ContainsKey(type))
                _referencePools.Add(type, new List<IReference>());
        }
    }
}
