using Cysharp.Threading.Tasks;
using Newtonsoft.Json;
using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using System.Threading.Tasks;
using UnityEngine;

namespace TFramework
{
    /// <summary>
    /// ļ
    /// </summary>
    public static class FileOperations
    {
        /// <summary>
        /// ·ݴб
        /// </summary>
        static List<string> paths = new List<string>();

        #region Json
        static JsonSerializerSettings settings = new JsonSerializerSettings
        {
            TypeNameHandling = TypeNameHandling.Auto, //  TypeNameHandling.All
            Formatting = Formatting.Indented,
            ReferenceLoopHandling = ReferenceLoopHandling.Ignore,
        };

        /// <summary>
        /// дJson
        /// </summary>
        /// <param name="data"></param>
        /// <param name="path">ļ·</param>
        public static void WriteJsonData(object data, string path)
        {
            string json = JsonConvert.SerializeObject(data, settings);
            WriteData(json, path);
        }

        /// <summary>
        /// 첽дJson
        /// </summary>
        /// <param name="data"></param>
        /// <param name="path">ļ·</param>
        public async static UniTask WriteJsonDataAsync(object data, string path)
        {
            string json = JsonConvert.SerializeObject(data, settings);
            await WriteDataAsync(json, path);
        }

        /// <summary>
        /// ȡJson
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="path">·</param>
        /// <returns></returns>
        public static T ReadJsonData<T>(string path)
        {
            if (FileExists(path))
                return JsonConvert.DeserializeObject<T>(ReadText(path), settings);
            else
                return default(T);
        }

        /// <summary>
        /// 첽ȡJson
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="path">·</param>
        /// <returns></returns>
        public static async UniTask<T> ReadJsonDataAsync<T>(string path)
        {
            if (FileExists(path))
                return JsonConvert.DeserializeObject<T>(await ReadTextAsync(path), settings);
            else
                return default(T);
        }
        #endregion

        /// <summary>
        /// д
        /// </summary>
        /// <param name="value">Ҫд</param>
        /// <param name="path">·</param>
        public static void WriteData(string value, string path)
        {
            //򿪻½ĵ
            using (StreamWriter sw = new StreamWriter(path))
            {
                //
                sw.WriteLine(value);
                //رĵ
                sw.Close();
                sw.Dispose();
            }
        }

        /// <summary>
        /// 첽д
        /// </summary>
        /// <param name="value">Ҫд</param>
        /// <param name="path">·</param>
        public static async UniTask WriteDataAsync(string value, string path)
        {
            //򿪻½ĵ
            using (StreamWriter sw = new StreamWriter(path))
            {
                //
                await sw.WriteAsync(value);
                //رĵ
                sw.Close();
                sw.Dispose();
            }
        }

        /// <summary>
        /// Զʽд
        /// </summary>
        /// <param name="obj"></param>
        /// <param name="path">·</param>
        public static void WriteBinaryData(System.Object obj, string path)
        {
            try
            {
                string folderPath = path.Remove(path.LastIndexOf('/'));
                if (!Directory.Exists(folderPath))
                    Directory.CreateDirectory(folderPath);
                BinaryFormatter binary = new BinaryFormatter();
                using (FileStream fs = File.Create(path))
                {
                    binary.Serialize(fs, obj);
                }
            }
            catch (Exception e)
            {
                Log.Error(e.Message);
            }

        }

        /// <summary>
        /// ȡļ
        /// </summary>
        /// <typeparam name="T">ļ</typeparam>
        /// <param name="path">ļ·</param>
        /// <returns></returns>
        public static T ReadBinaryData<T>(string path)
        {
            try
            {
                BinaryFormatter binary = new BinaryFormatter();
                using (FileStream fs = File.Open(path, FileMode.Open))
                {
                    return (T)binary.Deserialize(fs);
                }
            }
            catch (Exception e)
            {
                Log.Info(e.Message);
                return default(T);
            }
        }

        public static byte[] SafeReadAllBytes(string inFile)
        {
            try
            {
                if (string.IsNullOrEmpty(inFile) || !File.Exists(inFile))
                    return null;

                File.SetAttributes(inFile, FileAttributes.Normal);
                return File.ReadAllBytes(inFile);
            }
            catch (System.Exception ex)
            {
                Log.Error(string.Format("SafeReadAllBytes failed! path = {0} with err = {1}", inFile, ex.Message));
                return null;
            }
        }

        /// <summary>
        /// ȡı
        /// </summary>
        /// <param name="path">·</param>
        /// <returns></returns>
        public static string ReadText(string path)
        {
            if (!File.Exists(path)) return null;

            using (StreamReader str = new StreamReader(path, Encoding.UTF8))
            {
                if (str == null) return null;
                return str.ReadToEnd();
            }
        }

        /// <summary>
        /// 첽ȡı
        /// </summary>
        /// <param name="path">·</param>
        /// <returns></returns>
        public async static UniTask<string> ReadTextAsync(string path)
        {
            if (!File.Exists(path)) return null;

            using (StreamReader str = new StreamReader(path, Encoding.UTF8))
            {
                if (str == null) return null;
                return await str.ReadToEndAsync();
            }
        }

        /// <summary>
        /// ȡTexture2Dļ
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static Texture2D ReadTexture2D(string path)
        {
            if (!File.Exists(path))
                return null;
            FileStream fs = new FileStream(path, FileMode.Open);
            byte[] buffer = new byte[fs.Length];
            fs.Read(buffer, 0, buffer.Length);
            fs.Close();
            Texture2D originalTex = new Texture2D(2, 2);
            var iSLoad = originalTex.LoadImage(buffer);
            originalTex.Apply();
            if (iSLoad)
                return originalTex;
            else
                return null;
        }

        /// <summary>
        /// ȡļļ·
        /// </summary>
        /// <param name="dir">ļ·</param>
        /// <param name="excludeFile">Ҫųļĺ׺</param>
        /// <returns></returns>
        public static string[] GetFolderAllFilePath(string dir, params string[] excludeFile)
        {
            paths.Clear();
            FindFile(dir, excludeFile);
            return paths.ToArray();
        }

        static void FindFile(string dir, params string[] excludeFile)
        {
            DirectoryInfo info = new DirectoryInfo(dir);
            FileSystemInfo[] file = info.GetFileSystemInfos();
            foreach (var item in file)
            {
                if (item is DirectoryInfo)
                    FindFile(item.FullName, excludeFile);
                else
                {
                    if (excludeFile != null)
                    {
                        string suffix = item.Extension;
                        bool isAdd = true;

                        foreach (var su in excludeFile)
                        {
                            if (suffix == su)
                                isAdd = false;
                        }
                        if (isAdd)
                            paths.Add(item.FullName);
                    }
                    else
                        paths.Add(item.FullName);
                }
            }
        }

        /// <summary>
        /// ȡ·ļ·
        /// </summary>
        /// <param name="path">·</param>
        /// <returns></returns>
        public static string[] GetPathAllFolders(string path)
        {
            paths.Clear();
            FindFolders(path);
            return paths.ToArray();
        }

        static void FindFolders(string path)
        {
            DirectoryInfo info = new DirectoryInfo(path);
            FileSystemInfo[] file = info.GetFileSystemInfos();
            foreach (var item in file)
            {
                if (item is DirectoryInfo)
                {
                    FindFolders(item.FullName);
                    paths.Add(item.FullName);
                }
            }
        }

        /// <summary>
        /// ļ
        /// </summary>
        /// <param name="sourceDirName"></param>
        /// <param name="destDirName"></param>
        /// <param name="isOverlay">Ƿ񸲸ͬļ</param>
        public static void CopyDirectory(string sourceDirName, string destDirName,bool isOverlay=false)
        {
            try
            {
                if (!Directory.Exists(destDirName))
                {
                    Directory.CreateDirectory(destDirName);
                    File.SetAttributes(destDirName, File.GetAttributes(sourceDirName));
                }

                if (destDirName[destDirName.Length - 1] != Path.DirectorySeparatorChar)
                    destDirName = destDirName + Path.DirectorySeparatorChar;

                string[] files = Directory.GetFiles(sourceDirName);
                foreach (string file in files)
                {
                    if (File.Exists(destDirName + Path.GetFileName(file))&&!isOverlay)
                            continue;
                    FileInfo fileInfo = new FileInfo(file);
                    if (fileInfo.Extension.Equals(".meta", StringComparison.CurrentCultureIgnoreCase))
                        continue;

                    File.Copy(file, destDirName + Path.GetFileName(file), true);
                    File.SetAttributes(destDirName + Path.GetFileName(file), FileAttributes.Normal);
                }

                string[] dirs = Directory.GetDirectories(sourceDirName);
                foreach (string dir in dirs)
                {
                    CopyDirectory(dir, destDirName + Path.GetFileName(dir),isOverlay);
                }
            }
            catch (Exception ex)
            {
                throw ex;
            }
        }

        /// <summary>
        /// ļ
        /// </summary>
        /// <param name="sourceFileNmae"></param>
        /// <param name="destFlieName"></param>
        public static void CopyFile(string sourceFileNmae,string destFlieName)
        {
            FileInfo file = new FileInfo(sourceFileNmae);
            if (file.Exists)
                file.CopyTo(destFlieName, true);
            else
                Log.Error($"Ŀļ{sourceFileNmae}ڣ");
        }

        /// <summary>
        /// ļ·Ƿ
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static bool FileExists(string path) => File.Exists(path);

        /// <summary>
        /// ļǷ
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static bool FolderExists(string path) => Directory.Exists(path);

        /// <summary>
        /// ȡļָļϢ
        /// </summary>
        /// <param name="folder">ļ</param>
        /// <param name="suffix">׺</param>
        /// <returns></returns>
        public static FileInfo[] GetFiles(string folder, string suffix)
        {
            if (!Directory.Exists(folder)) return null;

            DirectoryInfo di = new DirectoryInfo(folder);
            return di.GetFiles().ToList().Where(p => p.Extension == suffix).Select(p => p).ToArray();
        }

        /// <summary>
        /// ȡļָļϢ
        /// </summary>
        /// <param name="folder">ļ</param>
        /// <param name="suffix">׺</param>
        /// <returns></returns>
        public static FileInfo[] GetFiles(string folder, string suffix, IComparer comparer)
        {
            if (!Directory.Exists(folder)) return null;

            DirectoryInfo di = new DirectoryInfo(folder);
            FileInfo[] fileInfo = di.GetFiles().ToList().Where(p => p.Extension == suffix).Select(p => p).ToArray();
            Array.Sort(fileInfo, comparer);
            return fileInfo;
        }

        /// <summary>
        /// ʱ
        /// </summary>
        /// <returns></returns>
        public static IComparer SortToTime => new FileTimeComparer();

        public class FileTimeComparer : IComparer
        {
            public int Compare(object x, object y)
            {
                FileInfo file1 = x as FileInfo;
                FileInfo file2 = y as FileInfo;
                return file1.LastWriteTime.CompareTo(file2.LastWriteTime);
            }
        }

        public static void CreateFloder(string path) => Directory.CreateDirectory(path);

        /// <summary>
        /// Ŀ·ļ
        /// </summary>
        /// <param name="path"></param>
        public static void CreateAbsoluteFloder(string path)
        {
            CreateFloder(GetAbsolutePath(path));
        }

        /// <summary>
        /// ȡĿ·ľ·
        /// </summary>
        /// <param name="path"></param>
        /// <returns></returns>
        public static string GetAbsolutePath(string path)
        {
            string _path = Application.dataPath.Remove(Application.dataPath.LastIndexOf('/') + 1);
            return _path + path;
        }
    }
}
