using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;
using System;

namespace TFramework
{
    [DefaultExecutionOrder(-999)]
    public class InputManager : BaseManager
    {
        /// <summary>
        /// Ƿ豸
        /// </summary>
        [SerializeField]
        private bool m_isEnableInputDevice = true;

        [SerializeField]
        private string m_inputDevice;

        public int m_checkError=1;

        private Dictionary<string, V_Axis> _v_axes = new Dictionary<string, V_Axis>();
        private Dictionary<string, V_Key> _v_keys = new Dictionary<string, V_Key>();

        private InputDeviceBase _inputDevice;
        [SerializeField]
        private float _doubleClickInterval = 0.2f;
        /// <summary>
        /// ˫ʱ
        /// </summary>
        public float DoubleClickInterval { get=>_doubleClickInterval; set=>_doubleClickInterval=value; }
        private IEnumerator _clickEnumerator;
        private float _currentKeyClickTime;
        private bool _startTime = false;

        /// <summary>
        /// 豸
        /// </summary>
        public InputDeviceBase InputDevice => _inputDevice = _inputDevice ?? GlobalTool.CretaInstanceToCurrentDomain<InputDeviceBase>(m_inputDevice);

        /// <summary>
        /// λ
        /// </summary>
        public Vector3 MousePosition { get; private set; }

        /// <summary>
        /// Ƿ豸
        /// </summary>
        public bool IsEnableInputDevice
        {
            get
            {
                return m_isEnableInputDevice;
            }
            set
            {
                m_isEnableInputDevice = value;
                if (!m_isEnableInputDevice)
                {
                    //InputDevice.OnShutdown();
                    ResetAll();
                }
                else
                    InputDevice.OnStartUp();
            }
        }

        private void Start()
        {
            if(IsEnableInputDevice)
                InputDevice?.OnStartUp();
        }

        public void Update()
        {
            if (IsEnableInputDevice)
                InputDevice?.OnRun();
        }

        /// <summary>
        /// Ƿס
        /// </summary>
        public bool IsAnyKey => IsEnableInputDevice && Input.anyKey;

        /// <summary>
        /// Ƿ
        /// </summary>
        public bool IsAnyKeyDown => IsEnableInputDevice && Input.anyKeyDown;

        /// <summary>
        /// Ƿ
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public bool IsExistV_Axis(string name) => _v_axes.ContainsKey(name);

        /// <summary>
        /// Ƿ
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public bool IsExistV_Key(string name) => _v_keys.ContainsKey(name);

        /// <summary>
        /// 
        /// </summary>
        /// <param name="name"></param>
        public void AddV_Axis(string name)
        {
            if (!IsExistV_Axis(name))
                _v_axes.Add(name, new V_Axis(name));
        }

        /// <summary>
        /// 
        /// </summary>
        /// <param name="name"></param>
        public void AddV_Key(string name)
        {
            if (!IsExistV_Key(name))
                _v_keys.Add(name, new V_Key(name));
        }

        /// <summary>
        /// Ƴ
        /// </summary>
        /// <param name="name"></param>
        public void RemoveV_Axis(string name)
        {
            if (!IsExistV_Axis(name))
                _v_axes.Remove(name);
            else
                Log.Error($"᡾{name}!");
        }

        /// <summary>
        /// Ƴ
        /// </summary>
        /// <param name="name"></param>
        public void RemoveV_Key(string name)
        {
            if (IsExistV_Key(name))
                _v_keys.Remove(name);
            else
                Log.Error($"{name}");
        }

        private IEnumerator CklickTime(V_Key key)
        {
            while (_currentKeyClickTime > 0)
            {
                _currentKeyClickTime -= Time.deltaTime;
                yield return YieldWaitTool.YieldWaitForFixedUpdate();
            }
            _startTime = false;
            key.Click();
        }

        #region 

        /// <summary>
        /// ð
        /// </summary>
        /// <param name="name"></param>
        public void SetV_KeyDown(string name)
        {
            if (!IsExistV_Key(name))
                AddV_Key(name);
            _v_keys[name].Pressed(m_checkError);
        }

        /// <summary>
        /// ð̧
        /// </summary>
        /// <param name="name"></param>
        public void SetV_KeyUp(string name)
        {
            if (!IsExistV_Key(name))
                AddV_Key(name);
            _v_keys[name].Released(m_checkError);
            if (!_startTime)
            {
                _startTime = true;
                if (_clickEnumerator != null)
                {
                    StopCoroutine(_clickEnumerator);
                    _clickEnumerator = null;
                }
                _currentKeyClickTime = DoubleClickInterval;
                _clickEnumerator = CklickTime(_v_keys[name]);
                StartCoroutine(_clickEnumerator);
            }
        }

        /// <summary>
        /// ȡ
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public bool GetV_KeyDown(string name) => GetV_KeysDown(name);

        /// <summary>
        /// ȡ̧
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public bool GetV_KeyUp(string name) => GetV_KeysUp(name);

        /// <summary>
        /// ס
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public bool GetV_Key(string name) => GetV_Keys(name);
        
        /// <summary>
        /// 
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public bool GetV_KeyClick(string name)
        {
            if (!IsExistV_Key(name))
                AddV_Key(name);
            return _v_keys[name].GetClick;
        }

        /// <summary>
        /// ˫
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public bool GetV_KeyDobuleClick(string name)
        {
            if (!IsExistV_Key(name))
                AddV_Key(name);
            return _v_keys[name].GetDoubleClick;
        }
        
        /// <summary>
        /// ϼס
        /// </summary>
        /// <param name="names"></param>
        /// <returns></returns>
        public bool GetV_Keys(params string[] names) => GetV_KeyOp(0, names);

        /// <summary>
        /// ϼ
        /// </summary>
        /// <param name="names"></param>
        /// <returns></returns>
        public bool GetV_KeysDown(params string[] names) => GetV_KeyOp(-1, names);

        /// <summary>
        /// ϼ̧
        /// </summary>
        /// <param name="names"></param>
        /// <returns></returns>
        public bool GetV_KeysUp(params string[] names) => GetV_KeyOp(1, names);

        /// <summary>
        /// ȡ
        /// </summary>
        /// <param name="names"></param>
        /// <param name="op">1 ̧ 0 ס -1 </param>
        /// <returns></returns>
        private bool GetV_KeyOp(int op = 0, params string[] names)
        {
            for (int i = 0; i < names.Length; i++)
            {
                if (!IsExistV_Key(names[i]))
                    AddV_Key(names[i]);
            }
            //if (op == 1)
            //    return _v_keys.Where(p => names.Contains(p.Key)).Select(p => p.Value).All(p => p.GetKeyUp);
            //else if (op == 0)
            //    return _v_keys.Where(p => names.Contains(p.Key)).Select(p => p.Value).All(p => p.GetKey);
            //else
            //    return _v_keys.Where(p => names.Contains(p.Key)).Select(p => p.Value).All(p => p.GetKeyDown);
            
            if (op == 1)
            {
                for (int i = 0; i < names.Length; i++)
                {
                    if (!_v_keys[names[i]].GetKeyUp)
                        return false;
                }
                return true;
            }
            else if (op == 0)
            {
                for (int i = 0; i < names.Length; i++)
                {
                    if (!_v_keys[names[i]].GetKey)
                        return false;
                }
                return true;
            }
            else
            {
                for (int i = 0; i < names.Length; i++)
                {
                    if (!_v_keys[names[i]].GetKeyDown)
                        return false;
                }
                return true;
            }
        }

        /// <summary>
        /// ס ɴϼ
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public bool GetKeys(params KeyCode[] keyCodes) => keyCodes.All(p => Input.GetKey(p));

        /// <summary>
        /// ̧ ɴϼ
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public bool GetKeysUp(params KeyCode[] keyCodes) => keyCodes.All(p => Input.GetKeyUp(p));

        /// <summary>
        ///  ɴϼ
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public bool GetKeyDown(params KeyCode[] keyCodes) => keyCodes.All(p => Input.GetKeyDown(p));
        #endregion

        #region 
        /// <summary>
        /// ֵ
        /// </summary>
        /// <param name="name"></param>
        /// <param name="value">ֵ</param>
        public void SetV_Axis(string name, float value)
        {
            if (!IsExistV_Axis(name))
                AddV_Axis(name);
            _v_axes[name].Update(value);
        }

        /// <summary>
        /// ֵΪ0
        /// </summary>
        /// <param name="name"></param>
        public void SetV_AxisZero(string name) => SetV_Axis(name, 0);

        /// <summary>
        /// ֵΪ1
        /// </summary>
        /// <param name="name"></param>
        public void SetV_AxisPositive(string name) => SetV_Axis(name, 1);
        /// <summary>
        /// ֵΪ-1
        /// </summary>
        /// <param name="name"></param>
        public void SetV_AxisNegative(string name) => SetV_Axis(name, -1);

        /// <summary>
        /// ȡֵ
        /// </summary>
        /// <param name="name"></param>
        /// <param name="raw">Ƿȡֵ</param>
        /// <returns>ֵ</returns>
        public float GetAxis(string name, bool raw = false)
        {
            if (!IsExistV_Axis(name))
                AddV_Axis(name);
            return raw ? _v_axes[name].GetValueRaw : _v_axes[name].GetValue;
        }
        #endregion

        /// <summary>
        /// λ
        /// </summary>
        /// <param name="value"></param>
        public void SetVirtualMousePosition(Vector3 value) => MousePosition = value;

        /// <summary>
        /// ״̬
        /// </summary>
        public void ResetAll()
        {
            foreach (var item in _v_axes.Values)
                item.Update(0);
            foreach (var item in _v_keys.Values)
                item.Released();
        }

        /// <summary>
        /// 
        /// </summary>
        private sealed class V_Key
        {
            public string Name { get; private set; }
            public float ClickCount 
            { 
                get=>_clickCount; 
                private set
                {
                    _clickCount = value;
                }
            }

            private float _clickCount = 0;
            public int _pressedFrame = -5;
            private int _releasedFrame = -5;
            private bool _pressed = false;
            private int _clickFrame = -5;
            private int _doubleClickFrame = -5;

            public V_Key(string name) => Name = name;

            public void Pressed(int error=0)
            {
                if (_pressed)
                    return;

                _pressed = true;
                _pressedFrame = Time.frameCount+ error;
                ClickCount += 0.5f;
            }

            public void Released(int error = 0)
            {
                if (!_pressed)
                    return;

                _pressed = false;
                _releasedFrame = Time.frameCount+ error;
                ClickCount += 0.5f;
            }

            public void Click() 
            {
                if (_clickCount >= 2)
                    _doubleClickFrame = Time.frameCount;
                else if (_clickCount > 0 && _clickCount <= 2)
                    _clickFrame = Time.frameCount;
                ClickCount = 0;
            }

            /// <summary>
            /// ס
            /// </summary>
            public bool GetKey => _pressed;

            /// <summary>
            /// 
            /// </summary>
            public bool GetKeyDown => _pressedFrame == Time.frameCount;

            /// <summary>
            /// ̧
            /// </summary>
            public bool GetKeyUp => _releasedFrame == Time.frameCount;

            /// <summary>
            /// 
            /// </summary>
            public bool GetClick => _clickFrame == Time.frameCount;

            /// <summary>
            /// ˫
            /// </summary>
            public bool GetDoubleClick => _doubleClickFrame == Time.frameCount;
        }

        /// <summary>
        /// 
        /// </summary>
        private sealed class V_Axis
        {
            public string Name { get; private set; }

            private float _value;

            public V_Axis(string name) => Name = name;

            public void Update(float value) => _value = value;

            public float GetValue => _value;

            public float GetValueRaw => _value > 0 ? 1 : (_value < 0 ? -1 : 0);
        }
    }
}
