using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;

namespace TFramework
{
    public class ReferencePoolManager : BaseManager
    {
        private Dictionary<Type, ReferencePool> _referencePools = new Dictionary<Type, ReferencePool>();

        public int Limit { get; set; } = 100;

        /// <summary>
        /// ȡ
        /// </summary>
        /// <param name="type"></param>
        /// <returns></returns>
        public int GetRerenceCount(Type type)
        {
            if (_referencePools.ContainsKey(type))
                return _referencePools[type].Count;
            else
                throw new UnityException($"ȡʧܣó{type.FullName}");
        }

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <returns></returns>
        public T Spawn<T>() where T : class, IReference, new()
        {
            Type type = typeof(T);
            RegisterRefer(type);
            return _referencePools[type].Spawn<T>();
        }

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="T"></typeparam>
        public void Recycle<T>(T refer) where T : class, IReference, new()
        {
            Type type = refer.GetType();
            RegisterRefer(type);
            _referencePools[type].Recycle<T>(refer);
        }

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="refes">ü</param>
        public void Recycles<T>(List<T> refes) where T : class, IReference, new() => Recycles<T>(refes.ToArray());

        /// <summary>
        /// 
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="refes"></param>
        public void Recycles<T>(T[] refes) where T : class, IReference, new()
        {
            if (refes == null) return;
            Type type = typeof(T);
            RegisterRefer(type);
            for (int i = 0; i < refes.Length; i++)
                _referencePools[type].Recycle<T>(refes[i]);
        }

        /// <summary>
        /// ָó
        /// </summary>
        /// <param name="type"></param>
        public void Clear(Type type) => _referencePools[type].Clear();

        /// <summary>
        /// ó
        /// </summary>
        public void ClearAll()
        {
            foreach (var refer in _referencePools)
                refer.Value.Clear();
        }

        /// <summary>
        /// ע
        /// </summary>
        /// <param name="type"></param>
        private void RegisterRefer(Type type)
        {
            if (!_referencePools.ContainsKey(type))
                _referencePools.Add(type, new ReferencePool(Limit));
        }
    }
}
