using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

namespace TFramework
{
    public class ResourceManager : BaseManager
    {
        /// <summary>
        /// AssetBundleԴ嵥
        /// </summary>
        public AssetBundleManifest AssetBundleManifest { get; private set; }

        /// <summary>
        /// AssetBundleABơAB
        /// </summary>
        public Dictionary<string, AssetBundle> AssetBundles { get; private set; } = new Dictionary<string, AssetBundle>();

        /// <summary>
        /// ǰĳ
        /// </summary>
        public Scene CurrentScene { get; private set; }

        public void Awake()
        {
            CurrentScene= SceneManager.GetActiveScene();
        }

        public T LoadAsset<T>(LoadResHandle<T> resHandle) where T : UnityEngine.Object
        {
            return LoadAsset<T>(resHandle.ResourcePath, resHandle.AssetPath, resHandle.AssetBundleName, resHandle.LoadMode);
        }

        /// <summary>
        /// ͬԴ
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="resPath"></param>
        /// <param name="assetPath"></param>
        /// <param name="assetBundleName"></param>
        /// <param name="resLoadMode"></param>
        /// <returns></returns>
        public T LoadAsset<T>(string resPath, string assetPath, string assetBundleName = "", ResLoadMode resLoadMode = ResLoadMode.Resourece) where T : UnityEngine.Object
        {
            if (resLoadMode == ResLoadMode.Resourece)
                return Resources.Load<T>(resPath);
            else
            {
#if UNITY_EDITOR
                if (AssetConfig.RuntimeMode == UpdateMode.Editor)
                    return UnityEditor.AssetDatabase.LoadAssetAtPath<T>(assetPath);
#endif
                if (AssetBundleManifest == null)
                {
                    if (!AssetBundles.ContainsKey(AssetConfig.AssetBundleManifestName))
                        AssetBundles.Add(AssetConfig.AssetBundleManifestName,
                            AssetBundle.LoadFromFile(AssetConfig.AssetBundleRootPath + AssetConfig.AssetBundleManifestName));
                    AssetBundleManifest = AssetBundles[AssetConfig.AssetBundleManifestName].LoadAsset<AssetBundleManifest>("AssetBundleManifest");
                }
                string[] dependencies = AssetBundleManifest.GetAllDependencies(assetBundleName);
                for (int i = 0; i < dependencies.Length; i++)
                {
                    if (!AssetBundles.ContainsKey(dependencies[i]))
                        AssetBundles.Add(dependencies[i], AssetBundle.LoadFromFile(AssetConfig.AssetBundleRootPath + dependencies[i]));
                }
                if (!AssetBundles.ContainsKey(assetBundleName))
                    AssetBundles.Add(assetBundleName, AssetBundle.LoadFromFile(AssetConfig.AssetBundleRootPath + assetBundleName));
                return AssetBundles[assetBundleName].LoadAsset<T>(assetPath);
            }
        }

        /// <summary>
        /// ͬResourcesµԴ
        /// </summary>
        /// <typeparam name="T">Դ</typeparam>
        /// <param name="resPath">Դ·</param>
        /// <returns></returns>
        public T LoadAsset<T>(string resPath) where T : UnityEngine.Object => LoadAsset<T>(resPath, "", "");

        /// <summary>
        /// ͬAssetBundleԴ
        /// </summary>
        /// <typeparam name="T">Դ</typeparam>
        /// <param name="assetPath">Դ·</param>
        /// <param name="assetBundleName">AB</param>
        /// <returns></returns>
        public T LoadAsset<T>(string assetPath, string assetBundleName) where T : UnityEngine.Object => LoadAsset<T>("", assetPath, assetBundleName, ResLoadMode.AssetBundle);


        /// <summary>
        /// 첽Դ
        /// </summary>
        /// <typeparam name="T">Դ</typeparam>
        /// <param name="resPath">Դ·</param>
        /// <param name="onLoading">¼</param>
        /// <param name="onLoadDone">¼</param>
        /// <returns></returns>
        public Coroutine LoadAssetAsync<T>(LoadResHandle<T> resHandle) where T : UnityEngine.Object
        {
            return StartCoroutine(LoadAsync(resHandle));
        }

        private IEnumerator LoadAsync<T>(LoadResHandle<T> resHandle) where T : UnityEngine.Object
        {
            UnityEngine.Object asset = null;
            if (resHandle.LoadMode == ResLoadMode.Resourece)
            {
                ResourceRequest request = Resources.LoadAsync<T>(resHandle.ResourcePath);
                while (!request.isDone)
                {
                    resHandle.OnLoading?.Invoke(request.progress);
                    yield return null;
                }
                resHandle.OnLoading?.Invoke(1);
                if (!request.asset)
                    throw new UnityException($"ԴʧܣResourcesļвԴ {resHandle.ResourcePath}!");
                else
                    asset = request.asset;
            }
            else
            {
#if UNITY_EDITOR
                if (AssetConfig.RuntimeMode == UpdateMode.Editor)
                {
                    resHandle.OnLoading?.Invoke(1);
                    asset = UnityEditor.AssetDatabase.LoadAssetAtPath<T>(resHandle.AssetPath);
                    if (asset)
                        asset = asset as T;
                    else
                        throw new UnityException($"Դʧܣ·{resHandle.AssetPath}вԴ {resHandle.AssetPath}");
                }
                else
                {
                    //ȴԴļ
                    yield return LoadDependenciesAssetBundleAsync(resHandle.AssetBundleName);
                    yield return LoadAssetBundleAsync(resHandle.AssetBundleName, resHandle.OnLoading);
                    if (AssetBundles.ContainsKey(resHandle.AssetBundleName.ToLower()))
                    {
                        //asset = AssetBundles[resHandle.AssetBundleName].LoadAsset<T>(resHandle.AssetPath);
                        AssetBundleRequest request = AssetBundles[resHandle.AssetBundleName.ToLower()].LoadAssetAsync<T>(resHandle.AssetPath);
                        yield return request;
                        while (!request.isDone)
                        {
                            resHandle.OnLoading(request.progress / 2 + 0.5f);
                            yield return null;
                        }
                        asset = request.asset;
                        if (!asset)
                            throw new UnityException(string.Format("ԴʧܣAB {0} вԴ {1}", resHandle.AssetBundleName, resHandle.AssetPath));
                    }
                }
#else
                yield return LoadDependenciesAssetBundleAsync(resHandle.AssetBundleName);
                    yield return LoadAssetBundleAsync(resHandle.AssetBundleName, resHandle.OnLoading);
                    if (AssetBundles.ContainsKey(resHandle.AssetBundleName.ToLower()))
                    {
                        AssetBundleRequest request = AssetBundles[resHandle.AssetBundleName.ToLower()].LoadAssetAsync<T>(resHandle.AssetPath);
                        yield return request;
                        while(!request.isDone)
                        {
                            resHandle.OnLoading(request.progress / 2+0.5f);
                            yield return null;
                        }
                        asset = request.asset;
                        if (!asset)
                            throw new UnityException(string.Format("ԴʧܣAB {0} вԴ {1}", resHandle.AssetBundleName, resHandle.AssetPath));
                    }
#endif
            }

            if (asset)
            {
                resHandle.OnLoadOver?.Invoke(asset as T);
            }
            else
                resHandle.OnLoadOver?.Invoke(null);

        }

        /// <summary>
        /// 첽س(Resourceģʽ)
        /// </summary>
        /// <param name="scenePath">·</param>
        /// <param name="onLoading">¼</param>
        /// <param name="onLoadDone">¼</param>
        /// <returns></returns>
        public Coroutine LoadSceneAsync(string scenePath, TAction<float> OnLoading, TAction<Scene> OnLoadOver, LoadSceneMode loadSceneMode = LoadSceneMode.Single)
        {
            return StartCoroutine(LoadScene(scenePath, ResLoadMode.Resourece, "", OnLoading, OnLoadOver, loadSceneMode));
        }


        /// <summary>
        /// 첽س
        /// </summary>
        /// <param name="scenePath">·</param>
        /// <param name="onLoading">¼</param>
        /// <param name="onLoadDone">¼</param>
        /// <returns></returns>
        public Coroutine LoadSceneAsync(LoadResHandle<Scene> resHandle, LoadSceneMode loadSceneMode = LoadSceneMode.Single)
        {
            return StartCoroutine(LoadScene(resHandle.AssetPath,resHandle.LoadMode,resHandle.AssetBundleName,resHandle.OnLoading,resHandle.OnLoadOver, loadSceneMode));
        }

        private IEnumerator LoadScene(string scenePath, ResLoadMode loadMode ,string assetBundleName ,TAction<float> OnLoading,TAction<Scene> OnLoadOver, LoadSceneMode loadSceneMode)
        {
            if (SceneManager.GetSceneByPath(scenePath) == CurrentScene)
            {
                Debug.LogWarning($"{scenePath}Ѽأ");
                yield break;
            }
            if (loadMode == ResLoadMode.Resourece)
            {
                AsyncOperation ao = SceneManager.LoadSceneAsync(scenePath, loadSceneMode);
                while (!ao.isDone)
                {
                    OnLoading?.Invoke(ao.progress);
                    yield return null;
                }
            }
            else
            {
                yield return LoadDependenciesAssetBundleAsync(assetBundleName);
#if UNITY_EDITOR
                if (AssetConfig.RuntimeMode == UpdateMode.Editor)
                {
                    LoadSceneParameters parameters = new LoadSceneParameters()
                    {
                        loadSceneMode = LoadSceneMode.Single,
                        localPhysicsMode = LocalPhysicsMode.None
                    };
                    AsyncOperation ao = UnityEditor.SceneManagement.EditorSceneManager.LoadSceneAsyncInPlayMode(scenePath, parameters);
                    while (!ao.isDone)
                    {
                        OnLoading?.Invoke(ao.progress);
                        yield return null;
                    }
                }
                else
                {
                    yield return LoadAssetBundleAsync(assetBundleName, OnLoading);
                    AsyncOperation ao = SceneManager.LoadSceneAsync(scenePath, loadSceneMode);
                    while (!ao.isDone)
                    {
                        OnLoading?.Invoke(ao.progress);
                        yield return null;
                    }
                }
#else
                    yield return LoadAssetBundleAsync(assetBundleName, OnLoading);
                    AsyncOperation ao = SceneManager.LoadSceneAsync(scenePath, loadSceneMode);
                    while (!ao.isDone)
                    {
                        OnLoading?.Invoke(ao.progress);
                        yield return null;
                    }
#endif
            }
            OnLoading?.Invoke(1);
            OnLoadOver?.Invoke(SceneManager.GetSceneByPath(scenePath));
            CurrentScene = SceneManager.GetActiveScene();
            //SceneManager.SetActiveScene(CurrentScene);
        }

        /// <summary>
        /// 첽ڴ 
        /// </summary>
        /// <returns></returns>
        public Coroutine ClearMemoryAsync()
        {
            return StartCoroutine(ClearMemory());
        }

        /// <summary>
        /// ڴ棬ͷſڴ棨첽
        /// </summary>
        /// <returns>Э̵</returns>
        private IEnumerator ClearMemory()
        {
            yield return Resources.UnloadUnusedAssets();
            GC.Collect();
            GC.WaitForPendingFinalizers();
        }

        /// <summary>
        /// 첽жԴ
        /// </summary>
        /// <returns></returns>
        public Coroutine UnLoadAsstAsync(ResLoadMode resLoadMode, string assetBundleName, bool unloadAllLoadedObjects = false)
        {
            return StartCoroutine(UnLoadAsst(resLoadMode, assetBundleName, unloadAllLoadedObjects));
        }

        private IEnumerator UnLoadAsst(ResLoadMode resLoadMode, string assetBundleName, bool unloadAllLoadedObjects = false)
        {
            yield return UnLoadAsset(ResLoadMode.Resourece, assetBundleName, unloadAllLoadedObjects);
        }

        /// <summary>
        /// 첽жس
        /// </summary>
        /// <param name="scene"></param>
        /// <returns></returns>
        public Coroutine UnloadSceneAsync(Scene scene, TAction<float> OnUnloading = null, TAction OnUnloadOver = null)
        {
            return StartCoroutine(UnloadScene(scene, OnUnloading, OnUnloadOver));
        }

        private IEnumerator UnloadScene(Scene scene, TAction<float> OnUnloading, TAction OnUnloadOver)
        {
            AsyncOperation ao = SceneManager.UnloadSceneAsync(scene);
            while (!ao.isDone)
            {
                OnUnloading?.Invoke(ao.progress);
                yield return null;
            }
            OnUnloadOver?.Invoke();
            CurrentScene = SceneManager.GetActiveScene();

        }

        public IEnumerator UnLoadAsset(ResLoadMode resLoadMode, string assetBundleName, bool unloadAllLoadedObjects = false)
        {
            if (resLoadMode == ResLoadMode.Resourece)
            {
                yield return Resources.UnloadUnusedAssets();
            }
            else
            {
                if (AssetBundles.ContainsKey(assetBundleName))
                {
                    AssetBundles[assetBundleName].Unload(unloadAllLoadedObjects);
                    AssetBundles.Remove(assetBundleName);
                }
                yield return null;
            }
        }

        /// <summary>
        /// 첽AB嵥
        /// </summary>
        /// <returns>Э̵</returns>
        private IEnumerator LoadAssetBundleManifestAsync()
        {
            if (string.IsNullOrEmpty(AssetConfig.AssetBundleManifestName))
            {
                throw new UnityException("Դļ AssetBundleManifestName ԣΪABṩ嵥");
            }
            else
            {
                if (AssetBundleManifest == null)
                {
                    yield return LoadAssetBundleAsync(AssetConfig.AssetBundleManifestName, null);

                    if (AssetBundles.ContainsKey(AssetConfig.AssetBundleManifestName))
                    {
                        AssetBundleManifest = AssetBundles[AssetConfig.AssetBundleManifestName].LoadAsset<AssetBundleManifest>("AssetBundleManifest");
                        UnLoadAsset(ResLoadMode.AssetBundle, AssetConfig.AssetBundleManifestName);
                    }
                }
            }
            yield return null;
        }

        /// <summary>
        /// 첽ABṩȻص
        /// </summary>
        /// <param name="assetBundleName">AB</param>
        /// <param name="onLoading">¼</param>
        /// <param name="isManifest">ǷǼ嵥</param>
        /// <returns>Э̵</returns>
        private IEnumerator LoadAssetBundleAsync(string assetBundleName, TAction<float> onLoading)
        {
            assetBundleName = assetBundleName.ToLower();
            if (!AssetBundles.ContainsKey(assetBundleName))
            {
                AssetBundleCreateRequest abRequst = AssetBundle.LoadFromFileAsync(AssetConfig.AssetBundleRootPath + assetBundleName);

                yield return abRequst;

                while (!abRequst.isDone)
                {
                    onLoading?.Invoke(abRequst.progress / 2);
                    yield return null;
                }

                if (abRequst.assetBundle)
                    AssetBundles.Add(assetBundleName, abRequst.assetBundle);
                else
                    throw new UnityException($" δصAB{assetBundleName}");
            }
            onLoading?.Invoke(1);
        }

        /// <summary>
        /// 첽AB
        /// </summary>
        /// <param name="assetBundleName">AB</param>
        /// <returns>Э̵</returns>
        private IEnumerator LoadDependenciesAssetBundleAsync(string assetBundleName)
        {
#if UNITY_EDITOR
            if (AssetConfig.RuntimeMode != UpdateMode.Editor)
            {
                yield return LoadAssetBundleManifestAsync();

                if (AssetBundleManifest != null)
                {
                    string[] dependencies = AssetBundleManifest.GetAllDependencies(assetBundleName);
                    foreach (string item in dependencies)
                    {
                        if (AssetBundles.ContainsKey(item))
                            continue;

                        yield return LoadAssetBundleAsync(item, null);
                    }
                }
            }
#else
                yield return LoadAssetBundleManifestAsync();

                if (AssetBundleManifest != null)
                {
                    string[] dependencies = AssetBundleManifest.GetAllDependencies(assetBundleName);
                    foreach (string item in dependencies)
                    {
                        if (AssetBundles.ContainsKey(item))
                        {
                            continue;
                        }

                        yield return LoadAssetBundleAsync(item,null);
                    }
                }
#endif
            yield return null;
        }
    }
}
