using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.UI;

namespace TFramework
{
    public interface IPanelData:IReference { }

    public class PanelData : IPanelData
    {
        public void Reset()
        {
            
        }
    }

    public class BasePanel
    {
        /// <summary>
        /// UIʵ
        /// </summary>
        public GameObject UIEntity { get; set; }

        protected Dictionary<string, UIBehaviour> uiComponents;

        public virtual bool StickyTopic { get; protected set; } = false;

        public bool IsOpen { get; protected set; }

        /// <summary>
        /// ʼ UIʵ屻ʱִ
        /// </summary>
        public virtual void OnInit()
        {
            uiComponents = new Dictionary<string, UIBehaviour>();
            GetAllUIBehaviour();
        }

        /// <summary>
        /// ʾʱ
        /// </summary>
        public virtual void OnEnter(IPanelData args = null)
        {
            IsOpen = true;
        }

        /// <summary>
        /// ͣ ֮
        /// </summary>
        public virtual void OnPause(IPanelData args = null)
        {

        }

        /// <summary>
        /// 
        /// </summary>
        public virtual void OnResume(IPanelData args = null)
        {

        }

        /// <summary>
        /// ˳ ر
        /// </summary>
        public virtual void OnExit(IPanelData args = null)
        {
            IsOpen = false;
        }

        /// <summary>
        /// ȡUIBehaviour
        /// </summary>
        private void GetAllUIBehaviour()
        {
            foreach (var item in UIEntity.GetComponentsInChildren<UIBehaviour>(true))
            {
                if (uiComponents.ContainsKey(item.ID))
                    throw new UnityException($"ͬһPanel·ͬUIBehaviourID{item.ID}");
                else
                    uiComponents.Add(item.ID, item);
            }
        }

        /// <summary>
        /// ȡUIBehaviour
        /// </summary>
        /// <param name="uiId"></param>
        /// <returns></returns>
        protected UIBehaviour GetUIBehaviour(string uiId)
        {
            if (uiComponents.ContainsKey(uiId))
                return uiComponents[uiId];
            else
            {
                Log.Error($"{UIEntity.name}δҵIDΪ{uiId}UIBehaviour");
                return null;
            }
        }

        protected T GetComponentInUIID<T>(string uiId) where T: Component
        {
            return GetUIBehaviour(uiId)?.GetComponent<T>();
        }

        protected Component  GetComponentInUIID(Type type, string uiId)
        {
            return GetUIBehaviour(uiId)?.GetComponent(type);
        }

        /// <summary>
        /// л
        /// </summary>
        internal void ChangedWord()
        {
            foreach (var item in uiComponents.Values)
            {
                Text text = item.GetComponent<Text>();
                if (!text) continue;
                string str = Main.GetMagr<LanguageManager>()?.GetLanguageText(text.text);
                if (string.IsNullOrEmpty(str)) continue;
                text.text = str;
            }
        }
    }
}

