using System;
using System.Collections;
using System.Collections.Generic;
using System.IO;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Runtime.Serialization.Formatters.Binary;
using System.Text;
using UnityEngine;
using UnityEngine.EventSystems;

namespace TFramework
{
    public static class GlobalTool
    {

        /// <summary>
        /// ǰǷͣUGUIؼ
        /// </summary>
        /// <returns>Ƿ</returns>
        public static bool IsPointerOverUGUI()
        {
            if (EventSystem.current)
            {
#if UNITY_ANDROID && !UNITY_EDITOR
                if (Input.touchCount > 0)
                {
                    return EventSystem.current.IsPointerOverGameObject(Input.GetTouch(0).fingerId);
                }
                else
                {
                    return false;
                }
#else
                return EventSystem.current.IsPointerOverGameObject();
#endif
            }
            else
            {
                return false;
            }
        }

        #region 
        private static HashSet<string> RunTimeAssemblies = new HashSet<string>()
        {
            "Assembly-CSharp","UnityEngine", "UnityEngine.CoreModule", "UnityEngine.UI", "UnityEngine.PhysicsModule","UnityEngine.EventSystems","TMPro",
            "UnityEngine.Events","TFramework.RunTime","KeMingVR"
        };

        /// <summary>
        /// ӵǰʵ
        /// </summary>
        /// <typeparam name="T"></typeparam>
        /// <param name="fullName"></param>
        /// <returns></returns>
        public static T CretaInstanceToCurrentDomain<T>(string fullName)where T:class
        {
            foreach (var assembly in AppDomain.CurrentDomain.GetAssemblies())
                foreach (var type in assembly.GetTypes())
                    if (typeof(T).IsAssignableFrom(type))//жtypeǷ
                        if (type.IsClass && type.FullName == fullName)
                            return assembly.CreateInstance(type.FullName) as T;
            return null;
        }

        /// <summary>
        /// ӵǰȡָType
        /// </summary>
        /// <param name="typeFullName"></param>
        /// <returns></returns>
        public static Type GetTypeToCurrentDomain(string typeFullName)
        {
            Type type = null;
            foreach (var assembly in AppDomain.CurrentDomain.GetAssemblies())
            {
                type = assembly.GetType(typeFullName)?? assembly.GetTypes().Find(p=>p.Name==typeFullName);

                if (type != null)
                    return type;
            }
            return type;
        }

        /// <summary>
        /// ȡʱǰ
        /// </summary>
        /// <returns></returns>
        public static List<Type> GetTypesInRunTimeAssemblies()
        {
            List<Type> types = new List<Type>();
            Assembly[] assemblys = AppDomain.CurrentDomain.GetAssemblies();
            for (int i = 0; i < assemblys.Length; i++)
            {
                if (RunTimeAssemblies.Contains(assemblys[i].GetName().Name))
                {
                    types.AddRange(assemblys[i].GetTypes());
                }
            }
            return types;
        }

        /// <summary>
        /// ȡʱǰ
        /// </summary>
        /// <param name="filter">ɸѡ</param>
        /// <returns></returns>
        public static List<Type> GetTypesInRunTimeAssemblies(TFunc<Type, bool> filter)
        {
            List<Type> types = new List<Type>();
            Assembly[] assemblys = AppDomain.CurrentDomain.GetAssemblies();
            for (int i = 0; i < assemblys.Length; i++)
            {
                if (RunTimeAssemblies.Contains(assemblys[i].GetName().Name))
                {
                    Type[] ts = assemblys[i].GetTypes();
                    foreach (var t in ts)
                    {
                        if (filter(t))
                        {
                            types.Add(t);
                        }
                    }
                }
            }
            return types;
        }

        #endregion

        #region byte[]ת
        /// <summary>
        /// תΪbyte
        /// </summary>
        /// <param name="obj">ת</param>
        /// <returns>תbyte</returns>
        public static byte[] Object2Bytes(object obj)
        {
            byte[] buff = obj != null ? new byte[Marshal.SizeOf(obj)] : new byte[0];
            if(obj!=null)
            {
                IntPtr ptr = Marshal.UnsafeAddrOfPinnedArrayElement(buff, 0);
                Marshal.StructureToPtr(obj, ptr, true);
            }
            return buff;
        }

        /// <summary>
        /// byteתɶ
        /// </summary>
        /// <param name="buff">תbyte</param>
        /// <param name="typ">תɵ</param>
        /// <returns>תɺĶ</returns>
        public static object Bytes2Object(byte[] buff, Type type)
        {
            IntPtr ptr = Marshal.UnsafeAddrOfPinnedArrayElement(buff, 0);
            return Marshal.PtrToStructure(ptr, type);
        }

        public static Texture2D Bytes2Texture2D(byte[] buff)
        {
            Texture2D texture2D = new Texture2D(512, 512);
            texture2D.LoadImage(buff);
            return texture2D;
        }
        #endregion

        #region 
        /// <summary>
        /// ȡȫ
        /// </summary>
        /// <param name="superSize">ֱʵӱ</param>
        /// <param name="onCaptureOverEvent">ȡ¼</param>
        public static void CaptureScreenshot(int superSize,TAction<Texture2D> onCaptureOverEvent)
        {
            Main.Instance.StartCoroutine(RecordFrame(superSize, onCaptureOverEvent));
        }

        private static IEnumerator RecordFrame(int superSize, TAction<Texture2D> onCaptureOverEvent)
        {
            yield return YieldWaitTool.YieldWaitForEndOfFrame();
            onCaptureOverEvent?.Invoke(ScreenCapture.CaptureScreenshotAsTexture(superSize));
        }

        /// <summary>
        /// ȡָ
        /// </summary>
        /// <param name="camera">ͼ</param>
        /// <param name="rect">ȡ</param>
        /// <returns></returns>
        public static Texture2D CameraCapture(Camera camera,Rect rect)
        {
            RenderTexture render = new RenderTexture((int)rect.width, (int)rect.height, -1);
            camera.targetTexture = render;
            camera.Render();
            RenderTexture currentRT = RenderTexture.active;
            RenderTexture.active = render;
            Texture2D tex = new Texture2D((int)rect.width, (int)rect.height, TextureFormat.ARGB32, false);
            tex.ReadPixels(rect, 0, 0);
            tex.Apply();

            camera.targetTexture = null;
            RenderTexture.active = currentRT; ;
            GameObject.Destroy(render);
            return tex;
        }

        /// <summary>
        /// ȡĻָ
        /// </summary>
        /// <param name="rect">ȡ</param>
        /// <param name="onCaptureOverEvent">ȡ¼</param>
        public static void ScreenCaptureRect(Rect rect, TAction<Texture2D> onCaptureOverEvent)
        {
            Main.Instance.StartCoroutine(RecordFrame(rect, onCaptureOverEvent));
        }

        private static IEnumerator RecordFrame(Rect rect, TAction<Texture2D> onCaptureOverEvent)
        {
            yield return YieldWaitTool.YieldWaitForEndOfFrame();
            Texture2D tex = new Texture2D((int)rect.width, (int)rect.height, TextureFormat.ARGB32, false);
            tex.ReadPixels(rect, 0, 0);
            tex.Apply();
            onCaptureOverEvent?.Invoke(tex);
        }
        #endregion

        #region ͼת
        public static Sprite Texture2DToSprite(Texture2D texture2D) =>
            Sprite.Create(texture2D, new Rect(0, 0, texture2D.width, texture2D.height), Vector2.zero);

        public static Sprite TextureToSprite(Texture texture)
        {
            Texture2D texture2D = TextureToTexture2D(texture);

            return Texture2DToSprite(texture2D);
        }

        public static Texture2D TextureToTexture2D(Texture texture)
        {
            RenderTexture renderTexture = RenderTexture.GetTemporary(texture.width, texture.height, 32);
            Graphics.Blit(texture, renderTexture);
            Texture2D texture2D = RenderTextureToTexture2D(renderTexture);
            RenderTexture.ReleaseTemporary(renderTexture);
            return texture2D;
        }

        public static Texture2D RenderTextureToTexture2D(RenderTexture renderTexture)
        {
            RenderTexture currentRT = RenderTexture.active;
            RenderTexture.active = renderTexture;
            Rect rect = new Rect(0, 0, renderTexture.width, renderTexture.height);
            Texture2D texture2D = new Texture2D(renderTexture.width, renderTexture.height);
            texture2D.ReadPixels(rect, 0, 0);
            texture2D.Apply();
            RenderTexture.active = currentRT;
            return texture2D;
        }
        #endregion

        #region GameObject
        
        #endregion
    }
}
